@@ -30,18 +30,7 @@ class keyATMcov : virtual public keyATMmeta
       std::vector<int> topic_ids;
       std::vector<int> cov_ids;
 
-      double Lambda_current;
-      double llk_current;
-      double llk_proposal;
-      double diffllk;
-      double r, u;
-
       // Slice sampling
-      double start, end;
-      double previous_p, new_p;
-      double newlikelihood, slice_, current_lambda;
-      double store_loglik;
-      double newlambdallk;
       double val_min;
       double val_max;
     
@@ -60,8 +49,8 @@ class keyATMcov : virtual public keyATMmeta
     void initialize_specific() final;
 
     // Iteration
-    virtual void iteration_single(int &it);
-    void sample_parameters(int &it);
+    virtual void iteration_single(int it);
+    void sample_parameters(int it);
     void sample_lambda();
     void sample_lambda_mh();
     void sample_lambda_slice();
@@ -69,8 +58,8 @@ class keyATMcov : virtual public keyATMmeta
     virtual double loglik_total();
     double loglik_total_label();
 
-    double likelihood_lambda(int &k, int &t);
-    void proposal_lambda(int& k);
+    double likelihood_lambda(int k, int t);
+    void proposal_lambda(int k);
 };
 
 
