@@ -588,8 +588,7 @@ int keyATMmeta::sample_z_label(VectorXd &alpha, int z, int s,
 
 
 
-int keyATMmeta::sample_s(VectorXd &alpha, int z, int s,
-                 int w, int doc_id)
+int keyATMmeta::sample_s(int z, int s, int w, int doc_id)
 {
   int new_s;
   double numerator, denominator;
